var g_sliderVals1 = [];

var g_sliderMinX = 0;
var g_sliderW = 400;

function repositionSlider(slider, bank, idx, v) {
	v = Math.min(1, Math.max(0, v));
	var x = g_sliderMinX + g_sliderW*v;
	slider.style.left = x;
}
function setSliderFromDelta(slider, bank, idx, origV, dx) {
	var dv = dx / g_sliderW;
	var v = origV + dv;
	v = Math.min(1, Math.max(0, v));
	g_sliderVals1[idx] = v;
	repositionSlider(slider, bank, idx, v);	
}
function formatSliderValue(v) {
	return ""+(Math.round(v*1000)/1000);
}

var g_isPlaying = false;
var g_buttonLabels = [];

function applyState(state) {
	var obj;
	var arr;
	
	if ((obj = state['visibleControls'])) {
		var d = 'inline-block';
		document.getElementById("playPause").style.display = (obj['playPause']) ? d : 'none';
        document.getElementById("cueControls").style.display = (obj['cueControls']) ? 'block' : 'none';
		document.getElementById("button1").style.display = (obj['button1']) ? d : 'none';
		document.getElementById("button2").style.display = (obj['button2']) ? d : 'none';
		document.getElementById("button3").style.display = (obj['button3']) ? d : 'none';
		document.getElementById("sliders1").style.display = (obj['sliders1']) ? 'block' : 'none';
	}
	
	if ((obj = state['sliderBank1'])) {
		if ((arr = obj['values']) && arr.length !== undefined) {
			g_sliderVals1 = arr;
		
			var bankIndex = 1;
			for (var i = 0; i < arr.length; i++) {
				var v = arr[i];
				var slider = document.getElementById("slider_"+bankIndex+"_"+i);
				if (slider) repositionSlider(slider, bankIndex, i, v);

				var field = document.getElementById("sliderField_"+bankIndex+"_"+i);
				if (field) field.innerHTML = formatSliderValue(v);
			}
		
			var names = obj['names'];
			if (names && names.length !== undefined) {
				for (var i = 0; i < names.length; i++) {
					var label = document.getElementById("sliderLabel_"+bankIndex+"_"+i);
					if (label) label.innerHTML = (names[i]) ? names[i] : "Slider "+(i+1);
				}
			}
		}
	}
	if ((arr = state['customButtons']) && arr.length !== undefined) {
		for (var i = 0; i < arr.length; i++) {
            if (g_buttonLabels[i] === arr[i])
                continue;
            
			var el = document.getElementById("button"+(i+1));
			if ( !el)
                continue;
            
            g_buttonLabels[i] = arr[i];
			el.innerHTML = ""+arr[i];
		}
	}
    
    if ((obj = state['isPlaying'])) {
        var b = (obj === true);
        if (b !== g_isPlaying) {
            g_isPlaying = b;
            document.getElementById("playPause").innerHTML = (g_isPlaying) ? "Stop" : "Play";
        }
    }
    
    var currEl = document.getElementById("currentCueInfo");
    var nextEl = document.getElementById("nextCueInfo");
    
    if ((obj = state['currentCue'])) {
        var idx = obj['index'];
        var name = obj['name'];
        var desc = obj['description'];
        
        var html = "<b>Now showing cue #"+(idx+1)+":</b><br/>";
        html += "<b>"+name+"</b><br/>";
        html += (desc) ? desc : "";
        currEl.innerHTML = html;
    } else {
        currEl.innerHTML = "";
    }
    
    if ((obj = state['nextCue'])) {
        var name = obj['name'];
        var desc = obj['description'];        

        var html = "<b>Next cue:</b><br/>";
        html += "<b>"+name+"</b><br/>";
        html += (desc) ? desc : "";
        nextEl.innerHTML = html;
    } else {
        nextEl = innerHTML = "";
    }
}

function loadStateAsync() {
	var req = new XMLHttpRequest();
	req.onreadystatechange = function(ev) {
		if (req.readyState === 4) {
			if (req.status !== 200) {
				console.log("loadStateAsync: error sending request, status "+req.status);
			} else {
				var state = JSON.parse(req.responseText);
				if ( !state) {
					console.log("loadStateAsync: could not parse response");
				} else {
					applyState(state);
				}
			}
		}
	}
	req.open('GET', '/conduit/remotecontrol/status', true);
	req.send();
}


var g_pollIntv = -1;

window.onload = function() {
	loadStateAsync();

	g_pollIntv = setInterval(function() {
		loadStateAsync();
	}, 250);
    
    // bind buttons
    var buttonIds = [
        'playPause', 'nextCue',
        'button1', 'button2', 'button3',
    ];
    for (var i = 0; i < buttonIds.length; i++) {
        (function() {
            var butid = buttonIds[i];
            var button = document.getElementById(butid);
            if (button) button.addEventListener('mousedown', function(ev) {
                ev.preventDefault();
                sendButton(butid);
                button.className = "button-down";
                setTimeout(function() {
                           button.className = "button";
                           }, 300);
            });
        })();
    }
	
    // bind sliders
	var bankIndex = 1;
	for (var i = 0; i < 8; i++) {
		(function(){
			var bidx = bankIndex;
			var idx = i;
			var isDown = false;
			var slider = document.getElementById("slider_"+bidx+"_"+idx);
			var field = document.getElementById("sliderField_"+bidx+"_"+idx);
			var track = document.getElementById("sliderTrack_"+bidx+"_"+idx);
			var startX = 0, startY = 0;
			var startV;
			function sliderClicked(ev) {
				ev.preventDefault();
				startX = ev.screenX;
				startY = ev.screenY;
				startV = g_sliderVals1[idx];
				isDown = true;				
				document.body.addEventListener('mousemove', sliderDragged, false);
				document.body.addEventListener('mouseup', sliderReleased, false);
			}
			function sliderDragged(ev) {
				ev.preventDefault();
				if ( !isDown) return;
				var dx = ev.screenX - startX;
				setSliderFromDelta(slider, bidx, idx, startV, dx);
				sendSlider(bidx, idx);				
				field.innerHTML = formatSliderValue(g_sliderVals1[idx]);
			}
			function sliderReleased(ev) {
				ev.preventDefault();
				isDown = false;
				document.body.removeEventListener('mousemove', sliderDragged, false);
				document.body.removeEventListener('mouseup', sliderReleased, false);
			}
			slider.addEventListener('mousedown', sliderClicked, false);
			
			track.addEventListener('click', function(ev) {
				ev.preventDefault();

				var offx = 0;
				var e = track;
				while (e && e !== document) {
					offx += e.offsetLeft;
					e = e.offsetParent;
				}
				var x = ev.clientX - offx;
				var v = (x - 5) / (g_sliderW*0.99);
				v = Math.min(1, Math.max(0, v));
				g_sliderVals1[idx] = v;
				repositionSlider(slider, bidx, idx, v);	
				sendSlider(bidx, idx);
				field.innerHTML = formatSliderValue(g_sliderVals1[idx]);
			}, false);
		})();
	}
}


function sendSlider(bank, idx) {
	var data = {};
	var arr = [];
	for (var i = 0; i <= idx; i++) {
		arr.push((i != idx) ? null : g_sliderVals1[i]);
	}
	data['sliderBank'+bank] = { 'values': arr };
    sendCmdXHR('slider', data);
}

function sendButton(id) {
	var data = { 'clickedButtonId': id };
    sendCmdXHR('button', data);
}

function sendCmdXHR(cmd, data) {
    var json = JSON.stringify({cmd: cmd, data: data});
	var req = new XMLHttpRequest();
	req.onreadystatechange = function(ev) {
		if (req.readyState === 4) {
			if (req.status !== 200) {
				console.log("error sending cmd request, status "+req.status);
			} else {
				var state = JSON.parse(req.responseText);
				if (state) applyState(state);
			}
		}
	}
	req.open('POST', '/conduit/remotecontrol/cmd', true);
	req.setRequestHeader('Content-Type', 'application/json');
	req.send(json);
}

